@@ -15,6 +15,7 @@ class BaseDataModel:
 
 class MapModel(BaseDataModel):
     _default_primitive_type = dict
+    _num_required_fields = 0
 
     def __init__(self, **kwargs):
         super().__init__()
@@ -29,11 +30,15 @@ class MapModel(BaseDataModel):
     def dict(self) -> dict:
         return self.childrens_map if self.childrens_map is not None else self.kwargs
 
+    @property
+    def num_required_fields(self) -> int:
+        return self._num_required_fields
+
 
 class BaseItem:
     _table = None
     _database_path: Optional[List[DatabasePathElement]] = None
-    _dict_key_expected_type_: Optional[type] = None
+    _dict_key_expected_type: Optional[type] = None
     _dict_items_excepted_type: Optional[type or MapModel] = None
     # We set the _database_path as static, so that the assign_internal_mapping_from_class can setup the path only once,
     # only by having access to the inheritor class type, not even the instance. Yet, when set the _database_path
@@ -41,6 +46,7 @@ class BaseItem:
     # the paths of the others), but rather, it is statically set on the class that inherit from BaseField.
 
     def __init__(self, field_type: Optional[type] = Any, custom_default_value: Optional[Any] = None):
+        # todo: add a file_url field_type
         self._value = None
         self._query = None
 
@@ -64,13 +70,13 @@ class BaseItem:
 
     def validate_data(self, load_data_into_objects: bool) -> Optional[Any]:
         from StructNoSQL.validator import validate_data
-        validated_data = validate_data(
+        validated_data, valid = validate_data(
             value=self._value, item_type_to_return_to=self, load_data_into_objects=load_data_into_objects,
             expected_value_type=self._field_type, map_model=self.map_model,
             dict_items_excepted_type=self.dict_items_excepted_type, dict_excepted_key_type=self.dict_key_expected_type
         )
         self._value = validated_data
-        return validated_data
+        return validated_data, valid
 
     def populate(self, value: Any):
         self._value = value
@@ -124,13 +130,16 @@ class BaseItem:
 
 
 class BaseField(BaseItem):
-    def __init__(self, name: str, field_type: Optional[type] = None, required: Optional[bool] = False,
+    def __init__(self, name: str, field_type: Optional[Any] = None, required: Optional[bool] = False, not_modifiable: Optional[bool] = False,
                  custom_default_value: Optional[Any] = None, key_name: Optional[str] = None):
         super().__init__(field_type=field_type if field_type is not None else Any, custom_default_value=custom_default_value)
         self._name = name
         self._required = required
         self._key_name = None
 
+        if not_modifiable is True:
+            raise Exception(f"Not modifiable not yet implemented")
+
         if key_name is not None:
             if field_type == dict or type(field_type) == _GenericAlias:
                 self._key_name = key_name
@@ -165,13 +174,23 @@ class BaseField(BaseItem):
         if item is not None:
             return item
         else:
-            # print(key)
-            # print(self.dict_items_excepted_type.__dict__)
+            print(key)
+            print(self.dict_items_excepted_type.__dict__)
             if self._field_type == dict:
-                dict_expected_type_item = self.dict_items_excepted_type.__dict__.get(key, None)
-                if dict_expected_type_item is not None:
-                    return dict_expected_type_item
-        raise AttributeError()
+                if isinstance(self.dict_items_excepted_type, (list, tuple)):
+                    for expected_type in self.dict_items_excepted_type:
+                        dict_expected_type_item = expected_type.__dict__.get(key, None)
+                        if dict_expected_type_item is not None:
+                            return dict_expected_type_item
+                else:
+                    dict_expected_type_item = self.dict_items_excepted_type.__dict__.get(key, None)
+                    if dict_expected_type_item is not None:
+                        return dict_expected_type_item
+
+        raise AttributeError(message_with_vars(
+            message="Attribute missing from BaseField and its dict_item expected type.",
+            vars_dict={"attributeKey": key, "__dict__": self.__dict__}
+        ))
 
     def post(self, value: any):
         print(self._database_path)
@@ -210,7 +229,7 @@ class MapItem(BaseField):
 
     def __init__(self, model_type: type, parent_field: BaseField):
         super().__init__(name=None, field_type=dict, custom_default_value=dict())
-        self.model_type = model_type
+        self.map_model = model_type
 
         from StructNoSQL.table import make_dict_key_var_name, try_to_get_primitive_default_type_of_item
         element_key = make_dict_key_var_name(parent_field.key_name)
