@@ -4,7 +4,7 @@ from StructNoSQL.dynamodb.models import DatabasePathElement
 from StructNoSQL.fields import BaseField, MapModel, MapField, MapItem
 from StructNoSQL.practical_logger import message_with_vars
 from StructNoSQL.utils.process_render_fields_paths import process_and_get_fields_paths_objects_from_fields_paths, \
-    process_and_make_single_rendered_database_path
+    process_and_make_single_rendered_database_path, process_validate_data_and_make_single_rendered_database_path
 
 
 class DatabaseKey(str):
@@ -29,8 +29,7 @@ class BaseTable:
             self.model = data_model
         else:
             self.model = data_model()
-        class_variables = assign_internal_mapping_from_class(table=self, class_instance=self.model)
-        print(class_variables)
+        assign_internal_mapping_from_class(table=self, class_instance=self.model)
 
     def get_single_field_item_from_single_item(self, key_name: str, key_value: str, field_to_get: str, query_kwargs: Optional[dict] = None) -> Any:
         response_data = self.dynamodb_client.get_item_in_path_target(
@@ -74,20 +73,23 @@ class BaseTable:
                         if matching_field_path_object is not None:
                             if matching_field_path_object.database_path is not None:
                                 matching_field_path_object.populate(value=current_item_value)
-                                current_item[current_item_key] = matching_field_path_object.validate_data(load_data_into_objects=False)
+                                current_item[current_item_key], valid = matching_field_path_object.validate_data(load_data_into_objects=False)
             return response.items
         else:
             return None
 
     def set_update_one_field(self, key_name: str, key_value: str, target_field: str, value_to_set: Any,
                              index_name: Optional[str] = None, query_kwargs: Optional[dict] = None) -> bool:
-        response = self.dynamodb_client.set_update_data_element_to_map(
-            key_name=key_name, key_value=key_value, value=value_to_set,
-            target_path_elements=process_and_make_single_rendered_database_path(
-                field_path=target_field, fields_switch=self.fields_switch, query_kwargs=query_kwargs
-            )
+        validated_data, valid, target_path_elements = process_validate_data_and_make_single_rendered_database_path(
+            field_path=target_field, fields_switch=self.fields_switch, query_kwargs=query_kwargs, data_to_validate=value_to_set
         )
-        return True if response is not None else False
+        if valid is True and target_path_elements is not None:
+            response = self.dynamodb_client.set_update_data_element_to_map(
+                key_name=key_name, key_value=key_value, value=validated_data,
+                target_path_elements=target_path_elements
+            )
+            return True if response is not None else False
+        return False
 
     def set_update_multiple_fields(self):
         raise Exception(f"Not implemented")
@@ -120,11 +122,18 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
         current_path_elements = list()
     output_mapping = dict()
 
-    class_variables = dict()
-    if class_type is not None:
-        class_variables = class_type.__dict__
-    elif class_instance is not None:
-        class_variables = class_instance.__class__.__dict__
+    if class_type is None:
+        if class_instance is not None:
+            class_type = class_instance.__class__
+        else:
+            raise Exception(message_with_vars(
+                message="class_type or class_instance args must be passed "
+                        "to the assign_internal_mapping_from_class function"
+            ))
+
+    class_variables = class_type.__dict__
+    num_required_fields: Optional[list] = getattr(class_type, "_num_required_fields", None)
+    # A static variable of a type is not accessible in the __dict__, but it is in the attributes.
 
     for variable_key, variable_item in class_variables.items():
         current_field_path = "" if nested_field_path is None else f"{nested_field_path}"
@@ -143,6 +152,10 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                 variable_item._table = table
                 output_mapping[variable_key] = ""
 
+                if num_required_fields is not None:
+                    if variable_item.required is True:
+                        num_required_fields += 1
+
                 current_field_path += f"{variable_item.field_name}" if len(current_field_path) == 0 else f".{variable_item.field_name}"
                 table.fields_switch[current_field_path] = variable_item
 
@@ -166,6 +179,10 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
                 variable_item._database_path = [*current_path_elements, new_database_path_element]
                 variable_item._table = table
 
+                if num_required_fields is not None:
+                    if variable_item.required is True:
+                        num_required_fields += 1
+
                 current_field_path += f"{variable_item.field_name}" if len(current_field_path) == 0 else f".{variable_item.field_name}"
                 table.fields_switch[current_field_path] = variable_item
 
@@ -190,16 +207,9 @@ def assign_internal_mapping_from_class(table: BaseTable, class_instance: Optiona
         except Exception as e:
             print(e)
 
+    if num_required_fields is not None:
+        setattr(class_type, "_num_required_fields", num_required_fields)
+        # We need to set the attribute, because when we go the num_required_fields with the get_attr
+        # function, we did not get a reference to the attribute, but a copy of the attribute value.
+
     return output_mapping
-
-
-"""
-if __name__ == "__main__":
-    users_table.projects.query()
-    print(users_table.ProjectsModel.ProjectInfos.primaryUrl.post(value="Yolooooooo"))
-    print(users_table.__class__.__dict__)
-    # print(signature(users_table.ProjectsModel))
-    # print(users_table.ProjectsModel(projectName="yolo").projectName)
-    # print(users_table.__class__.__dict__)
-"""
-
